function ValidationEngine(ScreenService, ZhUtil) {

	var self = this;

	this.getMappedValidations = function() {
		return validationNameToValidationFunction;
	};

	this.hasValidation = function(validationName) {
		return validationNameToValidationFunction.hasOwnProperty(validationName);
	};

	this.registerValidation = function(validationName, validationFunctionName) {
		if (validationName && validationFunctionName) {
			if (!validationNameToValidationFunction[validationName]) {
				validationNameToValidationFunction[validationName] = validationFunctionName;
			} else {
				throw ScreenService.i18n("Validation __validationName__ is already mapped!");
			}
		} else {
			throw ScreenService.i18n("Invalid parameters");
		}
	};

	this.unregisterValidation = function(validationName) {
		if (validationName) {
			delete validationNameToValidationFunction[validationName];
		}
	};

	this.min = function(value, validationAttrs) {
		var limit = parseFloat(validationAttrs.limit);
		if (validationAttrs.thousands || validationAttrs.decimal) {
			value = value.replace(new RegExp('\\'+validationAttrs.thousands,'g'), '');
			value = value.replace(validationAttrs.decimal, '.');
		}
		value = parseFloat(value);
		return {
			"valid": limit <= value,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + validationAttrs.limit
		};
	};

	this.max = function(value, validationAttrs) {
		var limit = parseFloat(validationAttrs.value);
		if (validationAttrs.thousands || validationAttrs.decimal) {
			value = value.replace(new RegExp('\\'+validationAttrs.thousands,'g'), '');
			value = value.replace(validationAttrs.decimal, '.');
		}
		value = parseFloat(value);
		return {
			"valid": limit >= value,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Maximum value: ") + validationAttrs.value
		};
	};

	this.range = function(value, validationAttrs) {
		var maxLimit = this.max(value, {
			value: validationAttrs.max
		}).valid;
		var minLimit = this.min(value, {
			limit: validationAttrs.min
		}).valid;
		return {
			"valid": minLimit && maxLimit,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Value range between ") + validationAttrs.min + ScreenService.i18n(
				" and ") + validationAttrs.max
		};
	};

	this.maxLength = function(value, validationAttrs) {
		var limit = parseInt(validationAttrs.length);
		value = Util.isUndefinedOrNull(value)? "" : value;
		return {
			"valid": limit >= value.toString().length,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Maximum length: ") + validationAttrs.length
		};
	};

	this.minLength = function(value, validationAttrs) {
		var limit = parseInt(validationAttrs.length);
		value = Util.isUndefinedOrNull(value)? "" : value;
		return {
			"valid": limit <= value.toString().length,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum length: ") + validationAttrs.length
		};
	};

	this.lengthRange = function(value, validationAttrs) {
		var maxLimit = this.maxLength(value, {
			length: validationAttrs.max
		}).valid;
		var minLimit = this.minLength(value, {
			length: validationAttrs.min
		}).valid;
		return {
			"valid": minLimit && maxLimit,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Length between ") + validationAttrs.min + ScreenService.i18n(" and ") +
				validationAttrs.max
		};
	};

	this.equalsTo = function(value, validationAttrs) {
		return {
			"valid": value === ZhUtil.evaluate(validationAttrs.text),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("The entered values do not match")
		};
	};

	this.required = function(value, validationAttrs) {
		var isDefined = !!value || Util.isNumber(value);
		return {
			"valid": !!(isDefined && (!Array.isArray(value) || value.length > 0)),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Required field")
		};
	};

	this.mail = function(value, validationAttrs) {
		var pattern = new RegExp(
			"^[_a-z0-9-]+(\\.[_a-z0-9-]+)*@[a-z0-9-]+(\\.[a-z0-9-]+)*(\\.[a-z]{2,4})$");
		return {
			"valid": !(value) || pattern.test(value),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Invalid email.")
		};
	};

	this.url = function(value, validationAttrs) {
		var protocols = ["http", "https", "ftp", "ssh"];
		var pattern = new RegExp("(" + protocols.join("|") +
			"?\\:\\/\\/)?([\\da-z\\.-]+)(\\:\\d)?(\\.([a-z\\.]{2,6})([\\/\\w \\.-]*)*\\/)?$");

		return {
			"valid": pattern.test(value),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Invalid URL.")
		};
	};

	this.dateIsValid = function(value, validationAttrs) {
		return {
			"valid": Util.isValidDate(value, validationAttrs ? validationAttrs.dateFormat : undefined),
			"message": ScreenService.i18n("Invalid date.")
		};
	};

	this.dateTimeIsValid = function(value, validationAttrs) {
		return {
			"valid": Util.isValidDateTime(value, validationAttrs ? validationAttrs.dateFormat : undefined),
			"message": ScreenService.i18n("Invalid date.")
		};
	};
	
	this.dateRange = function(value, validationAttrs) {
		var result = Util.isBetweenDate(validationAttrs.minimumDate, validationAttrs.maximumDate, value);
		return {
			"valid": result,
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Date range must be between ") + validationAttrs.minimumDate + ScreenService.i18n(
				" and ") + validationAttrs.maximumDate
		};
	};

	this.dateGreaterThan = function(value, validationAttrs) {
		return {
			"valid": Util.isDateGt(value, validationAttrs.minimumDate, validationAttrs ? validationAttrs.dateFormat : undefined),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("The current date must be greater than the previous date.")
		};
	};

	this.dateLessThan = function(value, validationAttrs) {
		return {
			"valid": Util.isDateLt(value, validationAttrs.maximumDate, validationAttrs ? validationAttrs.dateFormat : undefined),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("The current date must be smaller than the previous date.")
		};
	};

	this.hourGreaterThan = function(value, validationAttrs) {
		return {
			"valid": Util.isHourGt(value, validationAttrs.minimumHour, validationAttrs.dateFormat),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("This time must be greater than the previous")
		};
	};

	this.hourLessThan = function(value, validationAttrs) {
		return {
			"valid": Util.isHourLt(value, validationAttrs.maximumHour, validationAttrs.dateFormat),
			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("This time must be smaller than the previous.")
		};
	};

	this.formatArrayMessage = function(defaultMessage, name, index, lastIndex) {
		var message = defaultMessage + " \"" + name + "\"";
		if (index < lastIndex) {
			message += "\n";
		}
		return message;
	};

	this.extensions = function(value, validationAttrs) {
		var isValid = true;
		var message = "";
		if (value && Array.isArray(validationAttrs.value) && validationAttrs.value.length > 0) {
			var defaultMessage = ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("File with invalid extension.");
			var rgx = new RegExp('(' + validationAttrs.value.join('|').replace(/\./g, '\\.') + ')$');
			var lastIndex = value.length - 1;
			value.forEach(function(file, index) {
				var validation = rgx.test(file.name);
				if (!validation) {
					isValid = false;
					message += self.formatArrayMessage(defaultMessage, file.name, index, lastIndex);
				}
				return validation;
			});
		}
		return {
			"valid": isValid,
			"message": message
		};
	};

	this.maxSize = function(value, validationAttrs) {
		var isValid = true;
		var message = "";
		if (value && !!validationAttrs.value) {
			var defaultMessage = ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Very large file.");
			var lastIndex = value.length - 1;
			value.forEach(function(file, index) {
				var validation = (file.size <= validationAttrs.value);
				if (!validation) {
					isValid = false;
					message += self.formatArrayMessage(defaultMessage, file.name, index, lastIndex);
				}
				return validation;
			});
		}
		return {
			"valid": isValid,
			"message": message
		};
	};

	this.minSize = function(value, validationAttrs) {
		var isValid = true;
		var message = "";
		if (value && !!validationAttrs.value) {
			var defaultMessage = ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Very small file.");
			var lastIndex = value.length - 1;
			value.forEach(function(file, index) {
				var validation = (file.size > validationAttrs.value);
				if (!validation) {
					isValid = false;
					message += self.formatArrayMessage(defaultMessage, file.name, index, lastIndex);
				}
				return validation;
			});
		}
		return {
			"valid": isValid,
			"message": message
		};
	};

	var validationNameToValidationFunction = {
		"min": self.min,
		"max": self.max,
		"range": self.range,
		"maxLength": self.maxLength,
		"minLength": self.minLength,
		"lengthRange": self.lengthRange,
		"equalsTo": self.equalsTo,
		"required": self.required,
		"mail": self.mail,
		"url": self.url,
		"dateIsValid": self.dateIsValid,
		"dateTimeIsValid": self.dateTimeIsValid,
		"dateGreaterThan": self.dateGreaterThan,
		"dateLessThan": self.dateLessThan,
		"hourGreaterThan": self.hourGreaterThan,
		"hourLessThan": self.hourLessThan,
		"extensions": self.extensions,
		"maxSize": self.maxSize,
		"minSize": self.minSize
	};

}